@@ -1,8 +1,12 @@
+from __future__ import annotations
+
+import contextlib
 import functools
 import glob
 import locale
 import sys
 from contextlib import contextmanager
+from datetime import datetime
 from datetime import timedelta
 from os.path import isdir
 
@@ -11,7 +15,7 @@ import click_log
 
 from todoman import exceptions
 from todoman import formatters
-from todoman.configuration import ConfigurationException
+from todoman.configuration import ConfigurationError
 from todoman.configuration import load_config
 from todoman.interactive import TodoEditor
 from todoman.model import Database
@@ -25,7 +29,7 @@ click_log.basic_config()
 def handle_error():
     try:
         yield
-    except exceptions.TodomanException as e:
+    except exceptions.TodomanError as e:
         click.echo(e)
         sys.exit(e.EXIT_CODE)
 
@@ -78,7 +82,12 @@ def _validate_date_param(ctx, param, val):
     try:
         return ctx.formatter.parse_datetime(val)
     except ValueError as e:
-        raise click.BadParameter(e)
+        raise click.BadParameter(e) from None
+
+
+def _validate_categories_param(ctx, param, val):
+    ctx = ctx.find_object(AppContext)
+    return ctx.formatter.parse_categories(val)
 
 
 def _validate_priority_param(ctx, param, val):
@@ -86,13 +95,13 @@ def _validate_priority_param(ctx, param, val):
     try:
         return ctx.formatter.parse_priority(val)
     except ValueError as e:
-        raise click.BadParameter(e)
+        raise click.BadParameter(e) from None
 
 
-def _validate_start_date_param(ctx, param, val):
+def _validate_start_date_param(ctx, param, val) -> tuple[bool, datetime] | None:
     ctx = ctx.find_object(AppContext)
     if not val:
-        return val
+        return None
 
     if len(val) != 2 or val[0] not in ["before", "after"]:
         raise click.BadParameter("Format should be '[before|after] [DATE]'")
@@ -103,7 +112,7 @@ def _validate_start_date_param(ctx, param, val):
         dt = ctx.formatter.parse_datetime(val[1])
         return is_before, dt
     except ValueError as e:
-        raise click.BadParameter(e)
+        raise click.BadParameter(e) from None
 
 
 def _validate_startable_param(ctx, param, val):
@@ -147,6 +156,14 @@ def validate_status(ctx=None, param=None, val=None) -> str:
 
 
 def _todo_property_options(command):
+    click.option(
+        "--category",
+        "-c",
+        multiple=True,
+        default=(),
+        callback=_validate_categories_param,
+        help="Task categories. Can be used multiple times.",
+    )(command)
     click.option(
         "--priority",
         default="",
@@ -176,6 +193,10 @@ def _todo_property_options(command):
         kw["todo_properties"] = {
             key: kw.pop(key) for key in ("due", "start", "location", "priority")
         }
+        # longform is singular since user can pass it multiple times, but
+        # in actuality it's plural, so manually changing for #cache.todos.
+        kw["todo_properties"]["categories"] = kw.pop("category")
+
         return command(*a, **kw)
 
     return command_wrap
@@ -260,8 +281,8 @@ def cli(click_ctx, colour, porcelain, humanize, config):
     ctx = click_ctx.ensure_object(AppContext)
     try:
         ctx.config = load_config(config)
-    except ConfigurationException as e:
-        raise click.ClickException(e.args[0])
+    except ConfigurationError as e:
+        raise click.ClickException(e.args[0]) from None
 
     if porcelain and humanize:
         raise click.ClickException(
@@ -290,7 +311,7 @@ def cli(click_ctx, colour, porcelain, humanize, config):
         if isdir(path) and not path.endswith("__pycache__")
     ]
     if len(paths) == 0:
-        raise exceptions.NoListsFound(ctx.config["path"])
+        raise exceptions.NoListsFoundError(ctx.config["path"])
 
     ctx.db = Database(paths, ctx.config["cache_path"])
 
@@ -315,13 +336,11 @@ def invoke_command(click_ctx, command):
     click_ctx.invoke(cli.commands[name], *args, **opts)
 
 
-try:  # pragma: no cover
+with contextlib.suppress(ImportError):
     import click_repl
 
     click_repl.register_repl(cli)
     click_repl.register_repl(cli, name="shell")
-except ImportError:
-    pass
 
 
 @cli.command()
@@ -359,7 +378,7 @@ def new(ctx, summary, list, todo_properties, read_description, interactive):
         todo.priority = default_priority
 
     for key, value in todo_properties.items():
-        if value:
+        if value is not None:
             setattr(todo, key, value)
     todo.summary = " ".join(summary)
 
@@ -404,7 +423,7 @@ def edit(ctx, id, todo_properties, interactive, raw):
 
     changes = False
     for key, value in todo_properties.items():
-        if value is not None:
+        if value is not None and value != []:
             changes = True
             setattr(todo, key, value)
 
@@ -548,7 +567,6 @@ def move(ctx, list, ids):
 @pass_ctx
 @click.argument("lists", nargs=-1, callback=_validate_lists_param)
 @click.option("--location", help="Only show tasks with location containg TEXT")
-@click.option("--category", help="Only show tasks with category containg TEXT")
 @click.option("--grep", help="Only show tasks with message containg TEXT")
 @click.option(
     "--sort",
@@ -568,6 +586,14 @@ def move(ctx, list, ids):
 @click.option(
     "--due", default=None, help="Only show tasks due in INTEGER hours", type=int
 )
+@click.option(
+    "--category",
+    "-c",
+    multiple=True,
+    default=(),
+    help="Only show tasks with specified categories.",
+    callback=_validate_categories_param,
+)
 @click.option(
     "--priority",
     default=None,
@@ -625,9 +651,9 @@ def list(ctx, *args, **kwargs):
     The following commands can further filter shown todos, or include those
     omited by default:
     """
-    hide_list = (len([_ for _ in ctx.db.lists()]) == 1) or (  # noqa: C416
-        len(kwargs["lists"]) == 1
-    )
+    hide_list = (len([_ for _ in ctx.db.lists()]) == 1) or (len(kwargs["lists"]) == 1)
+
+    kwargs["categories"] = kwargs.pop("category")
 
     todos = ctx.db.todos(**kwargs)
     click.echo(ctx.formatter.compact_multiple(todos, hide_list))
